#include "curltest.h"
#include "ui_curltest.h"

#include "ftpcurl.h"

#include "ftpdownloaditem.h"

#include <QDebug>

#include <QAction>

CurlTest::CurlTest(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::CurlTest)
{
    Ftpcurl::global_init();
    ui->setupUi(this);

    init();
}

CurlTest::~CurlTest()
{
    ftpThread.endThread();
    ftpThread2.endThread();
    ftpThread3.endThread();
    ftpDownUrl.endThread();
    ftpThread.wait(50);
    ftpThread2.wait(50);
    ftpThread3.wait(50);
    ftpDownUrl.wait(50);

    Ftpcurl::global_cleanup();
    delete ui;
}

void CurlTest::closeEvent(QCloseEvent *)
{
}

void CurlTest::init()
{
    connect(ui->btnDownUrl, SIGNAL(clicked()),
            this, SLOT(OnBtnDownUrl()));
    connect(ui->btnFtpTest, SIGNAL(clicked()),
            this, SLOT(OnBtnFtpTest()));
    connect(ui->btnClear, SIGNAL(clicked()),
            this, SLOT(OnBtnClear()));
    connect(ui->btnPause, SIGNAL(clicked()),
            this, SLOT(OnBtnPause()));

    connect(&this->ftpThread, SIGNAL(TranslateProgress(QString,quint32,quint32,double)),
            this, SLOT(OnTranslateState(QString,quint32,quint32,double)));
    connect(&this->ftpThread2, SIGNAL(TranslateProgress(QString,quint32,quint32,double)),
            this, SLOT(OnTranslateState(QString,quint32,quint32,double)));
    connect(&this->ftpThread3, SIGNAL(TranslateProgress(QString,quint32,quint32,double)),
            this, SLOT(OnTranslateState(QString,quint32,quint32,double)));
    connect(&this->ftpDownUrl, SIGNAL(TranslateProgress(QString,quint32,quint32,double)),
            this, SLOT(OnTranslateState(QString,quint32,quint32,double)));
    connect(&this->ftpThread, SIGNAL(TranslateStop(QString,bool)),
            this, SLOT(OnTranslateStop(QString,bool)));
    connect(&this->ftpThread2, SIGNAL(TranslateStop(QString,bool)),
            this, SLOT(OnTranslateStop(QString,bool)));
    connect(&this->ftpThread3, SIGNAL(TranslateStop(QString,bool)),
            this, SLOT(OnTranslateStop(QString,bool)));
    connect(&this->ftpDownUrl, SIGNAL(TranslateStop(QString,bool)),
            this, SLOT(OnTranslateStop(QString,bool)));

    ui->tbvFtpDownloadList->setModel(&this->modFtpDownload);

    ui->tbvFtpDownloadList->setContextMenuPolicy(Qt::CustomContextMenu);

    this->acPause = this->meuFileList.addAction("Pause");
    this->acRemove = this->meuFileList.addAction("Remove");
    connect(this->acPause, SIGNAL(triggered()),
            this, SLOT(OnAcPause()));
    connect(this->acRemove, SIGNAL(triggered()),
            this, SLOT(OnAcRemove()));

    connect(ui->tbvFtpDownloadList, SIGNAL(customContextMenuRequested(QPoint)),
            this, SLOT(OnFileListMenu(QPoint)));

    ui->tbvFtpDownloadList->setItemDelegate(&this->dgtFtpDownloadProgressBar);

    ui->tbvFtpDownloadList->setSelectionBehavior(QAbstractItemView::SelectRows);
    ui->tbvFtpDownloadList->horizontalHeader()->setStretchLastSection(true);
    ui->tbvFtpDownloadList->verticalHeader()->hide();
    ui->tbvFtpDownloadList->setAlternatingRowColors(true);

    ftpThread.beginThread();
    ftpThread2.beginThread();
    ftpThread3.beginThread();
    ftpDownUrl.beginThread();


    FtpDownloadItem* downItem = new FtpDownloadItem(this);
    downItem->SetFileName("d:/First.exe");
    this->modFtpDownload.appendRowItem(downItem);
    ui->tbvFtpDownloadList->resizeRowsToContents();

    downItem = new FtpDownloadItem(this);
    downItem->SetFileName("d:/Next.exe");
    this->modFtpDownload.appendRowItem(downItem);
    ui->tbvFtpDownloadList->resizeRowsToContents();

    downItem = new FtpDownloadItem(this);
    downItem->SetFileName("d:/End.txt");
    this->modFtpDownload.appendRowItem(downItem);
    ui->tbvFtpDownloadList->resizeRowsToContents();

    downItem = new FtpDownloadItem(this);
    downItem->SetFileName("d:/tmp.tm");
    this->modFtpDownload.appendRowItem(downItem);
    ui->tbvFtpDownloadList->resizeRowsToContents();
}

void CurlTest::OnBtnDownUrl()
{
    QString url = ui->ledtUrl->text();
    if(url.isEmpty())
        return;

    qDebug() << url;

    this->ftpDownUrl.download(url,
                              "d:/tmp.tm");
}

void CurlTest::OnBtnFtpTest()
{
    if(ui->cbFirstFile->isChecked())
    {
        ftpThread.download("ftp://192.168.1.200/123.exe",
                           "d:/First.exe");
    }

    if(ui->cbNextFile->isChecked())
    {
        ftpThread2.download("ftp://192.168.1.200/FileZilla_Server-0_9_41.exe",
                           "d:/Next.exe");
    }

    if(ui->cbEndFile->isChecked())
    {
        ftpThread3.download(QString::fromLocal8Bit("ftp://192.168.1.200/.txt"),
                            "d:/End.txt");
    }
}

void CurlTest::OnBtnClear()
{
    //this->modFtpDownload.clear();
}

void CurlTest::OnBtnPause()
{
    this->ftpThread.pause();
    this->ftpThread2.pause();
    this->ftpThread3.pause();
    this->ftpDownUrl.pause();
}

void CurlTest::OnAcPause()
{
    QModelIndexList iList = ui->tbvFtpDownloadList->selectionModel()->selectedRows();
    QModelIndex index;
    foreach(index, iList)
    {
        QString fileName = this->modFtpDownload.data(
                    this->modFtpDownload.index(index.row(), 0)
                    ).toString();
        if(!fileName.compare("d:/First.exe"))
        {
            this->ftpThread.pause();
        }
        else if(!fileName.compare("d:/Next.exe"))
        {
            this->ftpThread2.pause();
        }
        else if(!fileName.compare("d:/End.txt"))
        {
            this->ftpThread3.pause();
        }
        else if(!fileName.compare("d:/tmp.tm"))
        {
            this->ftpDownUrl.pause();
        }
    }
}

void CurlTest::OnAcRemove()
{
    QModelIndexList iList = ui->tbvFtpDownloadList->selectionModel()->selectedRows();
    QModelIndex index;
    foreach(index, iList)
    {
        QString fileName = this->modFtpDownload.data(
                    this->modFtpDownload.index(index.row(), 0)
                    ).toString();
        if(!fileName.compare("d:/First.exe"))
        {
            this->ftpThread.pause();
            QFile file("d:/First.exe");
            file.remove();

        }
        else if(!fileName.compare("d:/Next.exe"))
        {
            this->ftpThread2.pause();
            QFile file("d:/Next.exe");
            file.remove();
        }
        else if(!fileName.compare("d:/End.txt"))
        {
            this->ftpThread3.pause();
            QFile file("d:/End.txt");
            file.remove();
        }
        else if(!fileName.compare("d:/tmp.tm"))
        {
            this->ftpDownUrl.pause();
            QFile file("d:/tmp.tm");
            file.remove();
        }
    }
}

void CurlTest::OnTranslateStop(QString fileName, bool bSuccess)
{
    this->modFtpDownload.resultState(fileName, bSuccess);
}

void CurlTest::OnTranslateState(QString fileName, quint32 fileSize, quint32 currentSize, double speed)
{
    this->modFtpDownload.updateState(fileName, fileSize, currentSize, speed);
}

void CurlTest::OnFileListMenu(QPoint pt)
{
    if(-1 == ui->tbvFtpDownloadList->rowAt(pt.y()))
        return;
    this->meuFileList.exec(QCursor::pos());
}
