#include "ftpcurl.h"
#include <QDebug>

#include <QTime>

#include <WinSock2.h>
#pragma comment(lib, "ws2_32.lib")

Ftpcurl::Ftpcurl(QString ftpurl, QString filePath, QObject *parent) :
    QObject(parent)
{
    curl = curl_easy_init();
    this->file.setFileName(filePath);
    this->setopt(ftpurl, this->file.size());
}

Ftpcurl::~Ftpcurl()
{
    if(this->file.isOpen())
        this->file.close();
    if(curl)
        curl_easy_cleanup(curl);
}

void Ftpcurl::setopt(QString url, quint32 offset)
{
    //curl_easy_setopt(this->curl, CURLOPT_URL, url.toUtf8().data());
    curl_easy_setopt(this->curl, CURLOPT_URL, url.toLocal8Bit().data());

    curl_easy_setopt(this->curl, CURLOPT_VERBOSE, 1);
    curl_easy_setopt(this->curl, CURLOPT_WRITEFUNCTION, procWrite);
    curl_easy_setopt(this->curl, CURLOPT_WRITEDATA, &file);

    curl_easy_setopt(this->curl, CURLOPT_RESUME_FROM, offset);
    sum = offset;

    curl_easy_setopt(this->curl, CURLOPT_NOPROGRESS, FALSE);
    curl_easy_setopt(this->curl, CURLOPT_PROGRESSFUNCTION, procProgress);
    curl_easy_setopt(this->curl, CURLOPT_PROGRESSDATA, this);

    curl_easy_setopt(this->curl, CURLOPT_NOSIGNAL, 1);

    tmBackup = QTime::currentTime();
}

CURLcode Ftpcurl::exec()
{
    return curl_easy_perform(this->curl);
}

CURL *Ftpcurl::CurlHandle() const
{
    return this->curl;
}

void Ftpcurl::EmitState(quint32 size, quint32 currentSize,
                        double speed)
{
    emit this->TranslateProgress(this->file.fileName(),
                                 size, currentSize,
                                 speed);
}

size_t Ftpcurl::procWrite(void *ptr, size_t size, size_t nmemb, void *ourpointer)
{
    QFile* pFile = reinterpret_cast<QFile*>(ourpointer);
    if(0 == pFile)
        return -1;
    if(!pFile->isOpen())
    {
        if(!pFile->open(QFile::WriteOnly | QFile::Append))
            return -2;
    }
    return pFile->write(reinterpret_cast<const char*>(ptr), size * nmemb);
}

int Ftpcurl::procProgress(void *pVoid, double t, double d, double /*ultotal*/, double /*ulnow*/)
{
    Ftpcurl* pCurl = reinterpret_cast<Ftpcurl*>(pVoid);

    QTime currentTm = QTime::currentTime();
    int var = pCurl->tmBackup.msecsTo(currentTm);
    pCurl->tmBackup.addMSecs(var);

    quint32 bytes = quint32(d);

    double speed = (bytes / (var / 1000.0));

    if(quint32(d) != 0)
    {
        if(t == d)
            speed = -1.0;
        pCurl->EmitState(pCurl->sum + t, pCurl->sum + d, speed);
    }
    else
    {
        if(t == pCurl->sum)
            speed = -1.0;
        pCurl->EmitState(t, pCurl->sum, speed);
    }
    return 0;
}

void Ftpcurl::global_init()
{
    curl_global_init(CURL_GLOBAL_WIN32);
}

void Ftpcurl::global_cleanup()
{
    curl_global_cleanup();
}



//////////////////////////////////////
void FtpcurlEx::global_init()
{
    Ftpcurl::global_init();
}

void FtpcurlEx::global_cleanup()
{
    Ftpcurl::global_cleanup();
}

FtpcurlEx::FtpcurlEx(QObject * /*parent*/) :
    bPause(false)
{
    this->curlm = curl_multi_init();
}

FtpcurlEx::~FtpcurlEx()
{
    if(this->curlm)
        curl_multi_cleanup(this->curlm);
}

void FtpcurlEx::appendTask(QString ftpurl, QString filePath)
{
    Ftpcurl* ftpTask = new Ftpcurl(ftpurl, filePath);
    if(0 != ftpTask)
    {
        this->setPause(false);
        connect(ftpTask, SIGNAL(TranslateProgress(QString,quint32,quint32,double)),
                this, SIGNAL(TranslateProgress(QString,quint32,quint32,double)));
        this->lstTask.append(ftpTask);
    }
}

void FtpcurlEx::pause()
{
    this->setPause(true);
}

FtpcurlEx::FtpcurlExcode FtpcurlEx::exec()
{
    FtpcurlExcode code = this->eOk;
    Ftpcurl* ftpTask;
    foreach(ftpTask, this->lstTask)
    {
        curl_multi_add_handle(this->curlm, ftpTask->CurlHandle());
    }

    int still_running = 1;

    while(CURLM_CALL_MULTI_PERFORM == curl_multi_perform(this->curlm, &still_running))
    {
    }
    timeoutCount = 0;   //ʱۼʼ
    while(still_running)
    {
        timeval timeout;
        timeout.tv_sec = 1;
        timeout.tv_usec = 0;

        fd_set fdread;
        fd_set fdwrite;
        fd_set fdexcept;
        FD_ZERO(&fdread);
        FD_ZERO(&fdwrite);
        FD_ZERO(&fdexcept);

        int maxfd = -1;
        curl_multi_fdset(curlm, &fdread, &fdwrite, &fdexcept, &maxfd);

        int rc = ::select(maxfd + 1, &fdread, &fdwrite, &fdexcept, &timeout);
        if(this->isPause())
            break;
        switch(rc)
        {
        case -1:
            qDebug() << "error:" << ::WSAGetLastError();
            break;
        case 0:
            ++timeoutCount;
            if(timeoutCount > 60)
            {
                code = this->eTimeout;
                break;
            }
        default:
            while(CURLM_CALL_MULTI_PERFORM == curl_multi_perform(curlm, &still_running))
            {
                if(this->isPause())
                    break;
                timeoutCount = 0;
            }
            break;
        }
        if(this->isPause() || this->eTimeout == code)
            break;
    }

    foreach(ftpTask, this->lstTask)
    {
        curl_multi_remove_handle(this->curlm, ftpTask->CurlHandle());
        delete ftpTask; //ѾͷŶ
    }
    this->lstTask.clear();
    return code;
}

bool FtpcurlEx::isPause()
{
    QMutexLocker lock(&this->muxPause);
    return this->bPause;
}

void FtpcurlEx::setPause(bool bPause)
{
    QMutexLocker lock(&this->muxPause);
    this->bPause = bPause;
}
