#include "ftpthread.h"

#include <QDebug>

//Task
Task::Task(Ftp::TaskID tid)
{
    this->tid = tid;
}

void Task::setDownLoadData(QString url, QString filePath)
{
    this->url = url;
    this->filePath = filePath;
}

Ftp::TaskID Task::taskID() const
{
    return this->tid;
}

FtpThread::FtpThread(QObject *parent) :
    QThread(parent)
{
    connect(&ftpEx, SIGNAL(TranslateProgress(QString,quint32,quint32,double)),
            this, SIGNAL(TranslateProgress(QString,quint32,quint32,double)));
}

void FtpThread::beginThread()
{
    this->setQuit(false);
    if(this->isRunning())
        return;
    this->start();
}

void FtpThread::endThread()
{
    this->setQuit(true);
    this->waitTask.wakeOne();
}

void FtpThread::download(QString url, QString filePath)
{
    if(this->eWaiteTask != this->State())
    {
        qDebug() << "busy";
        return;
    }
    Task* task = new Task(Ftp::eDownload);
    task->setDownLoadData(url, filePath);
    this->appendTask(task);
    this->waitTask.wakeOne();
}

void FtpThread::pause()
{
    this->ftpEx.pause();
}

void FtpThread::run()
{
    while(true)
    {
        QMutexLocker lock(&muxWait);
        this->setState(this->eWaiteTask);
        this->waitTask.wait(&this->muxWait);
        this->setState(this->eExecTask);

        Task* task = 0;
        while(task = this->firstTask())
        {
            switch(task->taskID())
            {
            case Ftp::eDownload:
                this->TaskDownLoad(task->url, task->filePath);
                break;
            }

            if(this->isQuit())
                break;
        }


        if(this->isQuit())
            break;
    }
}

void FtpThread::TaskDownLoad(QString url, QString filePath)
{
    FtpcurlEx::FtpcurlExcode code;
    qint32 reConnectCount = -1;
    bool bSuccess = true;
    do
    {
        ++reConnectCount;
        if(reConnectCount > 20)
        {//ﵽ, ȡ
            bSuccess = false;
            break;
        }
        ftpEx.appendTask(url, filePath);
        code = ftpEx.exec();
    }while(FtpcurlEx::eTimeout == code);
    emit this->TranslateStop(filePath, bSuccess);
}

FtpThread::ThreadState FtpThread::State()
{
    QMutexLocker lock(&this->muxState);
    return this->state;
}

void FtpThread::setState(FtpThread::ThreadState state)
{
    QMutexLocker lock(&this->muxState);
    this->state = state;
}

bool FtpThread::isQuit()
{
    QMutexLocker lock(&this->muxQuit);
    return this->bQuit;
}

void FtpThread::setQuit(bool bQuit)
{
    QMutexLocker lock(&this->muxQuit);
    this->bQuit = bQuit;
}

Task *FtpThread::firstTask()
{
    QMutexLocker lock(&this->muxTaskQueue);
    if(0 == this->lstTask.count())
        return 0;
    Task* task = this->lstTask.at(0);
    this->lstTask.removeAt(0);
    return task;
}

void FtpThread::appendTask(Task *task)
{
    QMutexLocker lock(&this->muxTaskQueue);
    this->lstTask.append(task);
}


