#include "modftpdownload.h"

#include "ftpdownloaditem.h"

ModFtpDownload::ModFtpDownload(QObject *parent) :
    QAbstractTableModel(parent)
{
}

int ModFtpDownload::rowCount(const QModelIndex &parent) const
{
    return this->lstRowItem.count();
}

int ModFtpDownload::columnCount(const QModelIndex &parent) const
{
    //ļ, ļС, ǰļС, ٶ, ؽ
    return 5;
}

QVariant ModFtpDownload::data(const QModelIndex &index, int role) const
{
    if(Qt::DisplayRole == role)
    {
        if(this->lstRowItem.count() < index.row() + 1)
            return QVariant();

        switch(index.column())
        {
        case 0: return this->lstRowItem.at(index.row())->FileName();
        case 1: return this->lstRowItem.at(index.row())->CurrentSize();
        case 2: return this->lstRowItem.at(index.row())->FileSize();
        case 3: {
                double speed = this->lstRowItem.at(index.row())->Speed();
                if(speed < 0) {
                    return QVariant();
                }else if(speed < 1024) {
                    return QString("%1 bytes/s").arg(
                                QString::number(speed, 'f', 2));
                } else if(speed < 1024 * 1024) {
                    return QString("%1 kb/s").arg(
                                QString::number(speed / 1024, 'f', 2));
                } else {
                    return QString("%1 mb/s").arg(
                                QString::number(speed / (1024 * 1024), 'f', 2));
                }
            }
        case 4: return this->lstRowItem.at(index.row())->Success() ? tr("online") : tr("offline");
        }
    }
    return QVariant();
}

void ModFtpDownload::appendRowItem(FtpDownloadItem *pRowItem)
{
    if(0 == pRowItem)
        return;
    this->lstRowItem.append(pRowItem);
    reset();
}

void ModFtpDownload::updateState(QString fileName, quint32 fileSize, quint32 currentSize,
                                 double speed)
{
    FtpDownloadItem* ftpItem;
    int i = -1;
    foreach(ftpItem, lstRowItem)
    {
        ++i;
        if(!ftpItem->FileName().compare(fileName))
        {
            ftpItem->SetFileSize(fileSize);
            ftpItem->SetCurrentSize(currentSize);
            ftpItem->SetSpeed(speed);
            emit this->dataChanged(this->index(i, 1),
                                   this->index(i, 3));
        }
    }
}

void ModFtpDownload::resultState(QString fileName, bool bSuccess)
{
    FtpDownloadItem* ftpItem;
    int i = -1;
    foreach(ftpItem, lstRowItem)
    {
        ++i;
        if(!ftpItem->FileName().compare(fileName))
        {
            ftpItem->SetSuccess(bSuccess);
            emit this->dataChanged(this->index(i, 4),
                                   this->index(i, 4));
        }
    }
}

void ModFtpDownload::clear()
{
    FtpDownloadItem* ftpItem;
    foreach(ftpItem, this->lstRowItem)
    {
        delete ftpItem;
    }
    this->lstRowItem.clear();
    reset();
}
